@@ -21,110 +21,53 @@ import collections
 import contextlib
 import functools
 import threading
+from typing import Optional
 
 from fasteners import _utils
 
 
-def read_locked(*args, **kwargs):
-    """Acquires & releases a read lock around call into decorated method.
-
-    NOTE(harlowja): if no attribute name is provided then by default the
-    attribute named '_lock' is looked for (this attribute is expected to be
-    a :py:class:`.ReaderWriterLock`) in the instance object this decorator
-    is attached to.
-    """
-
-    def decorator(f):
-        attr_name = kwargs.get('lock', '_lock')
-
-        @functools.wraps(f)
-        def wrapper(self, *args, **kwargs):
-            rw_lock = getattr(self, attr_name)
-            with rw_lock.read_lock():
-                return f(self, *args, **kwargs)
-
-        return wrapper
-
-    # This is needed to handle when the decorator has args or the decorator
-    # doesn't have args, python is rather weird here...
-    if kwargs or not args:
-        return decorator
-    else:
-        if len(args) == 1:
-            return decorator(args[0])
-        else:
-            return decorator
-
-
-def write_locked(*args, **kwargs):
-    """Acquires & releases a write lock around call into decorated method.
-
-    NOTE(harlowja): if no attribute name is provided then by default the
-    attribute named '_lock' is looked for (this attribute is expected to be
-    a :py:class:`.ReaderWriterLock` object) in the instance object this
-    decorator is attached to.
-    """
-
-    def decorator(f):
-        attr_name = kwargs.get('lock', '_lock')
-
-        @functools.wraps(f)
-        def wrapper(self, *args, **kwargs):
-            rw_lock = getattr(self, attr_name)
-            with rw_lock.write_lock():
-                return f(self, *args, **kwargs)
-
-        return wrapper
-
-    # This is needed to handle when the decorator has args or the decorator
-    # doesn't have args, python is rather weird here...
-    if kwargs or not args:
-        return decorator
-    else:
-        if len(args) == 1:
-            return decorator(args[0])
-        else:
-            return decorator
-
-
 class ReaderWriterLock(object):
-    """A reader/writer lock.
+    """An inter-thread readers writer lock."""
 
-    This lock allows for simultaneous readers to exist but only one writer
-    to exist for use-cases where it is useful to have such types of locks.
-
-    Currently a reader can not escalate its read lock to a write lock and
-    a writer can not acquire a read lock while it is waiting on the write
-    lock.
-
-    In the future these restrictions may be relaxed.
-
-    This can be eventually removed if http://bugs.python.org/issue8800 ever
-    gets accepted into the python standard threading library...
-    """
-
-    #: Writer owner type/string constant.
-    WRITER = 'w'
-
-    #: Reader owner type/string constant.
-    READER = 'r'
+    WRITER = 'w'  #: Writer owner type/string constant.
+    READER = 'r'  #: Reader owner type/string constant.
 
     def __init__(self,
                  condition_cls=threading.Condition,
-                 current_thread_functor=None):
+                 current_thread_functor=threading.current_thread):
+        """
+        Args:
+            condition_cls:
+                Optional custom `Condition` primitive used for synchronization.
+            current_thread_functor:
+                Optional function that returns the identity of the thread in case
+                threads are not properly identified by threading.current_thread
+        """
         self._writer = None
         self._pending_writers = collections.deque()
         self._readers = {}
         self._cond = condition_cls()
-        self._current_thread = threading.current_thread
+        self._current_thread = current_thread_functor
 
     @property
-    def has_pending_writers(self):
-        """Returns if there are writers waiting to become the *one* writer."""
+    def has_pending_writers(self) -> bool:
+        """Check if there pending writers
+
+        Returns:
+            Whether there are pending writers.
+        """
         return bool(self._pending_writers)
 
-    def is_writer(self, check_pending=True):
-        """Returns if the caller is the active writer or a pending writer."""
+    def is_writer(self, check_pending: bool = True) -> bool:
+        """Check if caller is a writer (optionally pending writer).
+
+        Args:
+            check_pending:
+                Whether to check for pending writer status.
+
+        Returns:
+            Whether the caller is the active (or optionally pending) writer.
+        """
         me = self._current_thread()
         if self._writer == me:
             return True
@@ -133,8 +76,22 @@ class ReaderWriterLock(object):
         else:
             return False
 
+    def is_reader(self) -> bool:
+        """Check if caller is a reader.
+
+        Returns:
+            Whether the caller is an active reader.
+        """
+        me = self._current_thread()
+        return me in self._readers
+
     @property
-    def owner(self):
+    def owner(self) -> Optional[str]:
+        """Caller ownership (if any) of the lock
+
+        Returns:
+            `'w'` if caller is a writer, `'r'` if caller is a reader, None otherwise.
+        """
         """Returns whether the lock is locked by a writer or reader."""
         if self._writer is not None:
             return self.WRITER
@@ -142,19 +99,14 @@ class ReaderWriterLock(object):
             return self.READER
         return None
 
-    def is_reader(self):
-        """Returns if the caller is one of the readers."""
-        me = self._current_thread()
-        return me in self._readers
-
     @contextlib.contextmanager
     def read_lock(self):
         """Context manager that grants a read lock.
 
         Will wait until no active or pending writers.
 
-        Raises a ``RuntimeError`` if a pending writer tries to acquire
-        a read lock.
+        Raises:
+            RuntimeError: if a pending writer tries to acquire a read lock.
         """
         me = self._current_thread()
         if me in self._pending_writers:
@@ -202,8 +154,8 @@ class ReaderWriterLock(object):
 
         Guaranteed for locks to be processed in fair order (FIFO).
 
-        Raises a ``RuntimeError`` if an active reader attempts to acquire
-        a lock.
+        Raises:
+            RuntimeError: if an active reader attempts to acquire a lock.
         """
         me = self._current_thread()
         i_am_writer = self.is_writer(check_pending=False)
@@ -231,22 +183,6 @@ class ReaderWriterLock(object):
                     self._cond.notify_all()
 
 
-@contextlib.contextmanager
-def try_lock(lock):
-    """Attempts to acquire a lock, and auto releases if acquired (on exit)."""
-    # NOTE(harlowja): the keyword argument for 'blocking' does not work
-    # in py2.x and only is fixed in py3.x (this adjustment is documented
-    # and/or debated in http://bugs.python.org/issue10789); so we'll just
-    # stick to the format that works in both (oddly the keyword argument
-    # works in py2.x but only with reentrant locks).
-    was_locked = lock.acquire(False)
-    try:
-        yield was_locked
-    finally:
-        if was_locked:
-            lock.release()
-
-
 def locked(*args, **kwargs):
     """A locking **method** decorator.
 
@@ -264,6 +200,10 @@ def locked(*args, **kwargs):
     NOTE(harlowja): a custom logger (which will be used if lock release
     failures happen) can be provided by passing a logger instance for keyword
     argument ``logger``.
+
+    NOTE(paulius): This function is DEPRECATED and will be kept until the end
+    of time. It is potentially used by oslo, but too specific to be recommended
+    for other projects
     """
 
     def decorator(f):
@@ -296,3 +236,103 @@ def locked(*args, **kwargs):
             return decorator(args[0])
         else:
             return decorator
+
+
+def read_locked(*args, **kwargs):
+    """Acquires & releases a read lock around call into decorated method.
+
+    NOTE(harlowja): if no attribute name is provided then by default the
+    attribute named '_lock' is looked for (this attribute is expected to be
+    a :py:class:`.ReaderWriterLock`) in the instance object this decorator
+    is attached to.
+
+    NOTE(paulius): This function is DEPRECATED and will be kept until the end
+    of time. It is potentially used by oslo, but too specific to be recommended
+    for other projects
+    """
+
+    def decorator(f):
+        attr_name = kwargs.get('lock', '_lock')
+
+        @functools.wraps(f)
+        def wrapper(self, *args, **kwargs):
+            rw_lock = getattr(self, attr_name)
+            with rw_lock.read_lock():
+                return f(self, *args, **kwargs)
+
+        return wrapper
+
+    # This is needed to handle when the decorator has args or the decorator
+    # doesn't have args, python is rather weird here...
+    if kwargs or not args:
+        return decorator
+    else:
+        if len(args) == 1:
+            return decorator(args[0])
+        else:
+            return decorator
+
+
+def write_locked(*args, **kwargs):
+    """Acquires & releases a write lock around call into decorated method.
+
+    NOTE(harlowja): if no attribute name is provided then by default the
+    attribute named '_lock' is looked for (this attribute is expected to be
+    a :py:class:`.ReaderWriterLock` object) in the instance object this
+    decorator is attached to.
+
+    NOTE(paulius): This function is DEPRECATED and will be kept until the end
+    of time. It is potentially used by oslo, but too specific to be recommended
+    for other projects
+    """
+
+    def decorator(f):
+        attr_name = kwargs.get('lock', '_lock')
+
+        @functools.wraps(f)
+        def wrapper(self, *args, **kwargs):
+            rw_lock = getattr(self, attr_name)
+            with rw_lock.write_lock():
+                return f(self, *args, **kwargs)
+
+        return wrapper
+
+    # This is needed to handle when the decorator has args or the decorator
+    # doesn't have args, python is rather weird here...
+    if kwargs or not args:
+        return decorator
+    else:
+        if len(args) == 1:
+            return decorator(args[0])
+        else:
+            return decorator
+
+
+@contextlib.contextmanager
+def try_lock(lock: threading.Lock) -> bool:
+    """Context manager that attempts to acquire a lock without a timeout, and
+    releases it on exit (if acquired).
+
+    Args:
+        lock:
+            A lock to try to acquire.
+
+    Returns:
+        Whether the lock was acquired.
+
+    # NOTE(harlowja): the keyword argument for 'blocking' does not work
+    # in py2.x and only is fixed in py3.x (this adjustment is documented
+    # and/or debated in http://bugs.python.org/issue10789); so we'll just
+    # stick to the format that works in both (oddly the keyword argument
+    # works in py2.x but only with reentrant locks).
+
+    NOTE(paulius): This function is DEPRECATED and will be kept until the end
+    of time. It is potentially used by oslo, but too specific to be recommended
+    for other projects
+    """
+    was_locked = lock.acquire(False)
+    try:
+        yield was_locked
+    finally:
+        if was_locked:
+            lock.release()
